using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace PKJ
{
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        float GameSpeed = 5;
        int Score;
        int HighScore;
        TimeSpan EndOfGameTimer;

        Player player;

        Texture2D groundTexture;
        Texture2D block;
        Texture2D enemyTexture;
        Texture2D coinTexture;
        Texture2D truck;

        List<Bullet> bulletList = new List<Bullet>();
        List<Ground> groundList = new List<Ground>();
        List<Enemy> enemyList = new List<Enemy>();
        List<Particle> particleList = new List<Particle>();
        List<Cloud> cloudList = new List<Cloud>();
        List<Coin> coinList = new List<Coin>();
        List<Platform> platformList = new List<Platform>();
        List<CoinParticles> coinParticles = new List<CoinParticles>();

        TimeSpan bulletShootRateTime = TimeSpan.FromSeconds(0.3f);
        TimeSpan previousBulletShootTime = TimeSpan.Zero;
        TimeSpan enemySpawnRateTime = TimeSpan.FromSeconds(0.7f);
        TimeSpan previousEnemySpawnTime = TimeSpan.Zero;
        TimeSpan coinSpawnRateTime = TimeSpan.FromSeconds(0.85f);
        TimeSpan previousCoinSpawnTime = TimeSpan.Zero;
        TimeSpan platformSpawnRateTime = TimeSpan.FromSeconds(0.85f);
        TimeSpan previousPlatformSpawnTime = TimeSpan.Zero;

        SoundEffect SoundCollectCoin;
        SoundEffect SoundShoot;
        Song Music;

        SpriteFont font;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            IsMouseVisible = true;
        }

        protected override void Initialize()
        {
            graphics.PreferredBackBufferWidth = 1024;
            graphics.PreferredBackBufferHeight = 768;
            graphics.IsFullScreen = true;
            graphics.ApplyChanges();
           

            base.Initialize();
        }

        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            Utils.init();

            groundTexture = Content.Load<Texture2D>("Textures/road");
            for (int i = 0; i < 10; i++)
			{
                groundList.Add(new Ground(groundTexture, new Vector2(i * 128 - 128, 568), 0));
			}

            player = new Player(Content.Load<Texture2D>("Textures/shooter"), new Vector2(150,500));
            enemyTexture = Content.Load<Texture2D>("Textures/zombie");

            //Block
            block = Content.Load<Texture2D>("Textures/block");
            coinTexture = Content.Load<Texture2D>("Textures/coins");
            truck = Content.Load<Texture2D>("Textures/truck");

            // Clouds
            cloudList.Add(new Cloud(Content.Load<Texture2D>("Textures/cloud3"), new Vector2(-118,27), 0.5f));
            cloudList.Add(new Cloud(Content.Load<Texture2D>("Textures/cloud3"), new Vector2(1083, 27), 0.5f));
            cloudList.Add(new Cloud(Content.Load<Texture2D>("Textures/cloud2"), new Vector2(160, 225), 1f));
            cloudList.Add(new Cloud(Content.Load<Texture2D>("Textures/cloud2"), new Vector2(1000, 225), 1f));
            cloudList.Add(new Cloud(Content.Load<Texture2D>("Textures/cloud1"), new Vector2(28, 412), 1.5f));
            cloudList.Add(new Cloud(Content.Load<Texture2D>("Textures/cloud1"), new Vector2(1000, 412), 1.5f));

            // Sound
            SoundCollectCoin = Content.Load<SoundEffect>("Audio/coin");
            SoundShoot = Content.Load<SoundEffect>("Audio/shoot");
            Music = Content.Load<Song>("Audio/walking_flat");
            MediaPlayer.Play(Music);

            font = Content.Load<SpriteFont>("gamefont");
        }

        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        protected override void Update(GameTime gameTime)
        {
                
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed || Keyboard.GetState().IsKeyDown(Keys.Escape))
                this.Exit();

            // Spawn time randomizer adjuster
            platformSpawnRateTime = TimeSpan.FromSeconds(Utils.randy.Next(10, 15));

            UpdateCoinParticles();
            UpdatePlatforms();
            UpdateCoin();
            UpdateEnemySpawnTime();
            UpdateBackground();
            UpdateGround();
            UpdateParticles();
            UpdateBullets();
            UpdateEnemies(gameTime);

            Spawn(gameTime, ref coinSpawnRateTime, ref previousCoinSpawnTime, "coin");
            Spawn(gameTime, ref enemySpawnRateTime, ref previousEnemySpawnTime, "enemy");
            Spawn(gameTime, ref bulletShootRateTime, ref previousBulletShootTime, "bullet");
            Spawn(gameTime, ref platformSpawnRateTime, ref previousPlatformSpawnTime, "truck");

            PlayerCoinCollisionCheck();
            BulletEnemyCollisionCheck();
            PlayerEnemyCollisionCheck();
            PlatformCollisions();

            player.Update();

            LimitGameSpeed();

            base.Update(gameTime);
        }

        private void UpdateCoinParticles()
        {
            for (int i = 0; i < coinParticles.Count; i++)
            {
                coinParticles[i].Update();
                if (coinParticles[i].removeFromWorld)
                {
                    coinParticles.RemoveAt(i);
                    i--;
                }
            }
        }

        private void PlatformCollisions()
        {
            int onPlatform = 0;
            for (int i = 0; i < platformList.Count; i++)
            {
                bool collision = player.DestRect.Intersects(platformList[i].PlatformRectangle) && player.Velocity.Y >= 0;
                if (collision)
                    onPlatform++;
            }
            if (onPlatform > 0)
                player.onPlatform = true;
            else
                player.onPlatform = false;
        }

        private void UpdatePlatforms()
        {
            for (int i = 0; i < platformList.Count; i++)
            {
                platformList[i].Update(GameSpeed);
            }
        }

        private void PlayerCoinCollisionCheck()
        {
            for (int i = 0; i < coinList.Count; i++)
            {
                if (player.DestRect.Intersects(coinList[i].DestRect))
                {
                    coinList[i].removeFromWorld = true;
                    Score++;
                }
            }
        }

        private void UpdateCoin()
        {
            for (int i = 0; i < coinList.Count; i++)
            {
                coinList[i].Update(GameSpeed);
                if (coinList[i].removeFromWorld)
                {
                    int s = 2; // speed
                    particleList.Add(new Particle(block, coinList[i].Position, s * -2, -1 * s, 20, new Color(253, 183, 6)));
                    particleList.Add(new Particle(block, coinList[i].Position, s * 2, 1 * s, 20, new Color(253, 183, 6)));
                    particleList.Add(new Particle(block, coinList[i].Position, s * -2, 1 * s, 20, new Color(253, 183, 6)));
                    particleList.Add(new Particle(block, coinList[i].Position, s * 3, -1 * s, 20, new Color(253, 183, 6)));
                    particleList.Add(new Particle(block, coinList[i].Position, s * -2 / 2, 2 + s / 2, 30, new Color(253, 183, 6)));
                    particleList.Add(new Particle(block, coinList[i].Position, s * 2 / 2, 2 + s / 2, 30, new Color(253, 183, 6)));
                    particleList.Add(new Particle(block, coinList[i].Position, s * -2 / 2, 2 + s / 2, 30, new Color(253, 183, 6)));
                    particleList.Add(new Particle(block, coinList[i].Position, s * 3 /2, 2 + s/2, 30, new Color(253, 183, 6)));

                    coinList.RemoveAt(i);
                    i--;

                    SoundCollectCoin.Play();
                }

            }
        }

        private void UpdateEnemySpawnTime()
        {
            if (GameSpeed < 10)
                enemySpawnRateTime = TimeSpan.FromSeconds(3f);
            else
                enemySpawnRateTime = TimeSpan.FromSeconds(0.2 * GameSpeed);
        }

        private void UpdateBackground()
        {
            for (int i = 0; i < cloudList.Count; i++)
            {
                cloudList[i].Update(GameSpeed);
            }
        }

        private void UpdateGround()
        {
            for (int i = 0; i < groundList.Count; i++)
            {
                groundList[i].Update(GameSpeed);

                if (groundList[i].RemoveFromWorld)
                {
                    groundList.RemoveAt(i);
                    i--;
                    groundList.Add(new Ground(groundTexture, new Vector2(1024 + 128, 568), 0));
                }
            }
        }

        private void UpdateParticles()
        {
            for (int i = 0; i < particleList.Count; i++)
            {
                particleList[i].Update();
                if (particleList[i].removeFromWorld)
                {
                    particleList.RemoveAt(i);
                    i--;
                }
            }
        }

        private void Spawn(GameTime gameTime, ref TimeSpan rate, ref TimeSpan previousTime, string type)
        {
            if (gameTime.TotalGameTime - previousTime > rate)
            {
                previousTime = gameTime.TotalGameTime;

                switch (type)
                {
                    case "enemy":
                        enemyList.Add(new Enemy(enemyTexture, new Vector2(1024, 500), gameTime));
                        break;
                    case "coin":
                        coinList.Add(new Coin(coinTexture, new Vector2(1024, 400), Utils.randy.Next(1,4)));
                        break;
                    case "bullet":
                        bulletList.Add(new Bullet(block, new Vector2(player.Position.X + 50, player.Position.Y + 50)));
                        bulletList.Add(new Bullet(block, new Vector2(player.Position.X + 64, player.Position.Y + 45)));
                        SoundShoot.Play();
                        break;
                    case "truck":
                        platformList.Add(new Platform(truck, new Vector2(1024, 459))); 
                        break;
                }

            }
        }

        private void UpdateBullets()
        {
            for (int i = 0; i < bulletList.Count; i++)
            {
                bulletList[i].Update();
                if (bulletList[i].removeFromWorld)
                {
                    bulletList.RemoveAt(i);
                    i--;
                }
            }
        }

        private void UpdateEnemies(GameTime gameTime)
        {
            for (int i = 0; i < enemyList.Count; i++)
            {
                enemyList[i].Update(gameTime, GameSpeed);
                if (enemyList[i].removeFromWorld)
                {
                    int s = 2; // speed
                    particleList.Add(new Particle(block, enemyList[i].Position, 1 * s, -1 * s, 40, Color.Red));
                    particleList.Add(new Particle(block, enemyList[i].Position, 2 * s, 1 * s, 40, Color.Red));
                    particleList.Add(new Particle(block, enemyList[i].Position, 1 * s, 1 * s, 40, Color.Red));
                    particleList.Add(new Particle(block, enemyList[i].Position, 3 * s, -1 * s, 40, Color.Red));
                    coinList.Add(new Coin(coinTexture, enemyList[i].Position, 1));
                    coinList.Add(new Coin(coinTexture, enemyList[i].Position, 2));

                    enemyList.RemoveAt(i);
                    i--;
                    GameSpeed++;
                }
            }
        }

        private void BulletEnemyCollisionCheck()
        {
            for (int i = 0; i < bulletList.Count; i++)
            {
                for (int j = 0; j < enemyList.Count; j++)
                {
                    if (bulletList[i].BulletRect.Intersects(enemyList[j].EnemyRect))
                    {
                        bulletList[i].removeFromWorld = true;
                        enemyList[j].HP--;
                        enemyList[j].HitTimer = 5;
                        int s = 2; // speed
                        particleList.Add(new Particle(block, bulletList[i].Position, 1 * s, -1 * s, 10, Color.Red));
                        particleList.Add(new Particle(block, bulletList[i].Position, 2 * s, 1 * s, 10, Color.Red));
                        particleList.Add(new Particle(block, bulletList[i].Position, 1 * s, 1 * s, 10, Color.Red));
                        particleList.Add(new Particle(block, bulletList[i].Position, 3 * s, -1 * s, 10, Color.Red));
                    }
                }
            }
        }

        private void PlayerEnemyCollisionCheck()
        {
            for (int i = 0; i < enemyList.Count; i++)
            {
                if (player.DestRect.Intersects(enemyList[i].EnemyRect))
                {
                    enemyList[i].removeFromWorld = true;
                    player.HitTimer = 5;
                    player.Position.X += -20;
                    player.Velocity.X = -20;
                    GameSpeed = 0;
                    for (int j = 0; j < Score; j++)
                    {
                        coinParticles.Add(new CoinParticles(coinTexture, new Vector2(player.Position.X, player.Position.Y), (float)Utils.randy.Next(-10, 10)));
                    }
                    Score = 0;
                }
            }
        }

        private void LimitGameSpeed()
        {
            // Max Game Speed
            if (GameSpeed > 50)
                GameSpeed = 50;
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(new Color(194, 198, 174));

            spriteBatch.Begin();
            for (int i = 0; i < cloudList.Count; i++)
            {
                cloudList[i].Draw(spriteBatch);
            }
            for (int i = 0; i < groundList.Count; i++)
            {
                groundList[i].Draw(spriteBatch);
            }
            for (int i = 0; i < platformList.Count; i++)
            {
                platformList[i].Draw(spriteBatch);
            }
            for (int i = 0; i < bulletList.Count; i++)
            {
                bulletList[i].Draw(spriteBatch);
            }

            for (int i = 0; i < enemyList.Count; i++)
            {
                enemyList[i].Draw(spriteBatch);
            }

            for (int i = 0; i < coinParticles.Count; i++)
            {
                coinParticles[i].Draw(spriteBatch);
            }
            for (int i = 0; i < particleList.Count; i++)
            {
                particleList[i].Draw(spriteBatch);
            }

            for (int i = 0; i < coinList.Count; i++)
            {
                coinList[i].Draw(spriteBatch);
            }

            player.Draw(spriteBatch);

            spriteBatch.DrawString(font, "Coins: " + Score.ToString(), new Vector2(100, 70), Color.White);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
