﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace PKJ
{
    class Player
    {
        public Vector2 Position;
        public Vector2 Velocity;
        public Vector2 MaxVelocity;
        const float GRAVITY = 0.6f;
        public Texture2D Sprite;
        public Rectangle SourceRect;
        public Rectangle DestRect;
        const int FRAMEWIDTH = 69;
        public int offset;
        public string State;
        public int HitTimer = 0;
        public bool onPlatform = false;
        Color color;

        // Keyboard States
        private KeyboardState currentKeyboardState;
        private KeyboardState previousKeyboardState;

        // Gamepad States
        private GamePadState currentGamepadState;
        private GamePadState previousGamepadState;

        public Player(Texture2D sprite, Vector2 position)
        {
            this.Position = position;
            this.State = "NORMAL";
            this.Sprite = sprite;
            this.offset = 0;
            this.Velocity = new Vector2(0, 0);
            this.MaxVelocity = new Vector2(0, 20);
            this.SourceRect = new Rectangle(this.offset * FRAMEWIDTH, 0, FRAMEWIDTH, this.Sprite.Height);
            this.DestRect = new Rectangle((int)Math.Round(this.Position.X), (int)Math.Round(this.Position.Y), FRAMEWIDTH, 95);
        }
        public void Update() {
            ReadInput();
            HitCheck();

            if (this.Position.Y > 500)
            {
                this.Velocity.Y = 0;
                this.Position.Y = 500;
                this.State = "NORMAL";
            }
            else if (this.Position.Y < 50)
                this.State = "FALL";

            if (this.State == "JUMP")
            {
                this.Velocity.Y = -this.MaxVelocity.Y;
                this.State = "FALL";
            }
            if (this.State == "FALL")
                this.Velocity.Y += GRAVITY;

            if (this.onPlatform)
                this.Velocity.Y = -this.MaxVelocity.Y / 2;

            this.Position.X += this.Velocity.X;
            this.Position.Y += this.Velocity.Y;

            // Lock player position
            this.Position.X = MathHelper.Clamp(this.Position.X, 69, 1024 - 100);

            // Lazy man's animation
            this.offset++;
            if (this.offset > 3*4) this.offset = 0;
            this.SourceRect.X = this.offset/4 * FRAMEWIDTH;

            // Update Rectangle
            this.DestRect.X = (int)Math.Round(this.Position.X);
            this.DestRect.Y = (int)Math.Round(this.Position.Y);
        }
        public void Draw(SpriteBatch spriteBatch) 
        {
            spriteBatch.Draw(this.Sprite, this.DestRect, this.SourceRect, this.color);
        }

        public void ReadInput()
        {
            // Save previous keyboard/gamepad states
            previousKeyboardState = currentKeyboardState;
            previousGamepadState = currentGamepadState;

            // Read current keyboard/gamepad
            currentKeyboardState = Keyboard.GetState();
            currentGamepadState = GamePad.GetState(PlayerIndex.One);

            // Bools for simpler looking conditions
            bool left = currentKeyboardState.IsKeyDown(Keys.A) || currentKeyboardState.IsKeyDown(Keys.Left) || currentGamepadState.ThumbSticks.Left.X < -0.5;
            bool right = currentKeyboardState.IsKeyDown(Keys.D) || currentKeyboardState.IsKeyDown(Keys.Right) || currentGamepadState.ThumbSticks.Left.X > 0.5;
            bool up = currentKeyboardState.IsKeyDown(Keys.W) || currentKeyboardState.IsKeyDown(Keys.Up) || currentGamepadState.ThumbSticks.Left.Y > 0.5;
            bool down = currentKeyboardState.IsKeyDown(Keys.S) || currentKeyboardState.IsKeyDown(Keys.Down) || currentGamepadState.ThumbSticks.Left.Y < -0.5;
            bool button1 = currentKeyboardState.IsKeyDown(Keys.Z) || currentGamepadState.Buttons.A > 0;
            bool button2 = currentKeyboardState.IsKeyDown(Keys.Space) || currentGamepadState.Buttons.B > 0;
            bool prevButton1 = previousKeyboardState.IsKeyDown(Keys.Z) || previousGamepadState.Buttons.A > 0;
            bool prevButton2 = previousKeyboardState.IsKeyDown(Keys.Space) || previousGamepadState.Buttons.B > 0;

            if ( ((button1 && !prevButton1) || (up && !previousKeyboardState.IsKeyDown(Keys.Up))) && (this.State != "JUMP" && this.State != "FALL"))
                this.State = "JUMP";

            if (left)
                this.Velocity.X = -10;

            if (right && this.Velocity.X > -10)
                this.Velocity.X = 10;
            
            if (this.Velocity.X < -10)
                this.Velocity.X += 1;

            if (!left || !right) {
                if (this.Velocity.X < 0)
                    this.Velocity.X += 1;
                else if (this.Velocity.X > 0)
                    this.Velocity.X += -1;
            }
        }

        public void HitCheck()
        {
            if (this.HitTimer != 0)
            {
                this.color = Color.Red;
                this.HitTimer--;
            }
            else
                this.color = Color.White;
        }
    }
}
